package com.example.mapbox;


import static com.mapbox.mapboxsdk.style.layers.PropertyFactory.iconImage;
import static com.mapbox.mapboxsdk.style.layers.PropertyFactory.iconOffset;

import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import androidx.activity.result.ActivityResultCallback;
import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.goong_mapbox.R;
import com.example.mapbox.adapter.SuggestionAdapter;
import com.example.mapbox.api.IApiService;
import com.example.mapbox.api.RetrofitInstance;
import com.example.mapbox.listener.OnItemClickListener;
import com.example.mapbox.model.AutoComplete;
import com.example.mapbox.model.Location;
import com.example.mapbox.model.Route;
import com.example.mapbox.response.AutoCompleteResponse;
import com.example.mapbox.response.DirectionResponse;
import com.example.mapbox.response.PlaceDetailResponse;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.mapbox.android.core.location.LocationEngineRequest;
import com.mapbox.android.core.permissions.PermissionsManager;
import com.mapbox.geojson.Feature;
import com.mapbox.geojson.FeatureCollection;
import com.mapbox.geojson.LineString;
import com.mapbox.geojson.Point;
import com.mapbox.geojson.Polygon;
import com.mapbox.mapboxsdk.Mapbox;
import com.mapbox.mapboxsdk.annotations.Icon;
import com.mapbox.mapboxsdk.annotations.IconFactory;
import com.mapbox.mapboxsdk.annotations.MarkerOptions;
import com.mapbox.mapboxsdk.camera.CameraPosition;
import com.mapbox.mapboxsdk.camera.CameraUpdateFactory;
import com.mapbox.mapboxsdk.geometry.LatLng;
import com.mapbox.mapboxsdk.geometry.LatLngBounds;
import com.mapbox.mapboxsdk.location.LocationComponent;
import com.mapbox.mapboxsdk.location.LocationComponentActivationOptions;
import com.mapbox.mapboxsdk.location.LocationComponentOptions;
import com.mapbox.mapboxsdk.location.modes.CameraMode;
import com.mapbox.mapboxsdk.location.modes.RenderMode;
import com.mapbox.mapboxsdk.maps.MapView;
import com.mapbox.mapboxsdk.maps.MapboxMap;
import com.mapbox.mapboxsdk.maps.OnMapReadyCallback;
import com.mapbox.mapboxsdk.maps.Style;
import com.mapbox.mapboxsdk.plugins.annotation.SymbolManager;
import com.mapbox.mapboxsdk.plugins.annotation.SymbolOptions;
import com.mapbox.mapboxsdk.style.layers.FillLayer;
import com.mapbox.mapboxsdk.style.layers.LineLayer;
import com.mapbox.mapboxsdk.style.layers.Property;
import com.mapbox.mapboxsdk.style.layers.PropertyFactory;
import com.mapbox.mapboxsdk.style.sources.GeoJsonSource;
import com.mapbox.mapboxsdk.plugins.annotation.Symbol;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class MainActivity extends AppCompatActivity implements OnMapReadyCallback {
    private MapView mapView;
    private MapboxMap mapboxMap;
    private RecyclerView recyclerView;

    private Button directionButton;

    private SymbolManager symbolManager;

    private SuggestionAdapter adapter;
    private List<AutoComplete> allSuggestions;

    private LatLng selectedLocation;

    private String symbolIconId = "symbolIconId";

    private String geojsonSourceLayerId = "geojsonSourceLayerId";

    private PermissionsManager permissionsManager;
    private LocationComponent locationComponent;

    private final ActivityResultLauncher<String> activityResultLauncher = registerForActivityResult(new ActivityResultContracts.RequestPermission(), new ActivityResultCallback<Boolean>() {
        @Override
        public void onActivityResult(Boolean result) {
            if (result) {
                Toast.makeText(MainActivity.this, "Permission granted!", Toast.LENGTH_SHORT).show();
            }
        }
    });

    public void onMapReady(@NonNull MapboxMap mapboxMap) {
        String uri = getResources().getString(R.string.goong_map_url) + "/assets/goong_map_web.json?api_key=" + getResources().getString(R.string.goong_map_key);
        mapboxMap.setStyle(
                new Style.Builder().fromUri(uri),
                new Style.OnStyleLoaded() {
                    @Override
                    public void onStyleLoaded(@NonNull Style style) {
                        LatLng start = new LatLng(21.029579719995272, 105.85242472181584);
                        LatLng end = new LatLng(20.9409074, 106.2832288);
                        symbolManager = new SymbolManager(mapView, mapboxMap, style);
                        symbolManager.setIconAllowOverlap(true);
                        symbolManager.setIconIgnorePlacement(true);
                        //Chỉ được sử dụng ảnh dạng bitmap, k phải ảnh vector trong TH custom marker
//                        IconFactory iconFactory = IconFactory.getInstance(MainActivity.this);
//                        Icon icon = iconFactory.fromResource(R.drawable.blue_marker_view);
                        mapboxMap.addMarker(new MarkerOptions()
                                .position(start));

                        mapboxMap.addMarker(new MarkerOptions()
                                .position(end)
                        );
                        CameraPosition position = new CameraPosition.Builder()
                                .target(start)
                                .zoom(15)
                                .tilt(20)
                                .build();
                        mapboxMap.animateCamera(CameraUpdateFactory.newCameraPosition(position), 1200);
                    }
                });
        this.mapboxMap = mapboxMap;
    }

    private void addSymbol(@NonNull Style style) {
        // Enable symbol manager icon allow overlap and icon ignore placement settings
        symbolManager.setIconAllowOverlap(true);
        symbolManager.setIconIgnorePlacement(true);
        Symbol symbol = symbolManager.create(new SymbolOptions()
                .withLatLng(new LatLng(21.029579719995272, 105.85242472181584))
                .withIconImage(symbolIconId)
                .withIconSize(2.0f)
                .withTextField("Hoan Kiem")
                .withTextOffset(new Float[]{0f, 2f})
                .withIconColor("red")
        );

        // Load an image for the marker
        style.addImage(symbolIconId, getResources().getDrawable(R.drawable.baseline_location_on_24));
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Mapbox.getInstance(this, getString(R.string.mapbox_access_token));
        setContentView(R.layout.activity_main);
        mapView = (MapView) findViewById(R.id.mapView);
        mapView.onCreate(savedInstanceState);
        mapView.getMapAsync(this::onMapReady);
        directionButton = findViewById(R.id.directionBtn);
        directionButton.setVisibility(View.GONE);
        EditText searchEditText = findViewById(R.id.search_edit_text);
        recyclerView = findViewById(R.id.recycler_view);
        recyclerView.setVisibility(View.VISIBLE);
        allSuggestions = new ArrayList<>();
        //Gán sự kiện khi click vào từng item
        this.adapter = new SuggestionAdapter(allSuggestions, new OnItemClickListener() {
            @Override
            public void onItemClick(int position, String placeId) {
                //TODO: Chuyển camera đến điểm đó và gán marker
                //Và chỉ đường từ điểm đó đến điểm mình search
                fetchDetailLocation(placeId);
                recyclerView.setVisibility(View.GONE);
                directionButton.setVisibility(View.VISIBLE);
            }
        });
        recyclerView.setLayoutManager(new LinearLayoutManager(this));
        recyclerView.setAdapter(adapter);

        searchEditText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                search(s.toString());
            }

            @Override
            public void afterTextChanged(Editable s) {
            }
        });

        searchEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (!hasFocus) {
                    // Perform actions when EditText loses focus (blurred)
                    // For example, hide the keyboard
                    InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                    imm.hideSoftInputFromWindow(searchEditText.getWindowToken(), 0);
                    recyclerView.setVisibility(View.GONE);
                } else {
                    // EditText gained focus, show the RecyclerView
                    recyclerView.setVisibility(View.VISIBLE);
                }
            }
        });

        directionButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Style style = mapboxMap.getStyle();
                LatLng start = new LatLng(21.029579719995272, 105.85242472181584);
                fetchDirections(start, selectedLocation, style);
            }
        });
    }

    private void search(String textSearch) {
        if (recyclerView != null)
            recyclerView.setVisibility(View.VISIBLE);
        List<AutoComplete> filteredList = new ArrayList<>();
        try {
            if (textSearch != null) {
                IApiService service = RetrofitInstance.getRetrofitInstance(getResources().getString(R.string.goong_api_url)).create(IApiService.class);
                Call<AutoCompleteResponse> call = service.getAutoComplete(textSearch, getResources().getString(R.string.goong_api_key));
                call.enqueue(new Callback<AutoCompleteResponse>() {
                    @Override
                    public void onResponse(Call<AutoCompleteResponse> call, Response<AutoCompleteResponse> response) {
                        if (response != null && response.isSuccessful()) {
                            AutoCompleteResponse data = response.body();
                            if (data != null && data.getPredictions() != null) {
                                List<AutoComplete> autoCompletes = data.getPredictions();
                                for (AutoComplete item : autoCompletes) {
                                    filteredList.add(item);
                                }
                                if (filteredList != null && MainActivity.this.adapter != null) {
                                    MainActivity.this.adapter.updateSuggestions(filteredList);
                                }
                            }
                        } else {
                            // Handle error
                        }
                    }

                    @Override
                    public void onFailure(Call<AutoCompleteResponse> call, Throwable t) {
                        Toast.makeText(MainActivity.this, "Auto complete is error!!!", Toast.LENGTH_SHORT).show();
                    }
                });
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        if (filteredList != null && this.adapter != null)
            this.adapter.updateSuggestions(filteredList);
    }

    private void fetchDetailLocation(String placeId) {
        Style style = mapboxMap.getStyle();
        IApiService service = RetrofitInstance.getRetrofitInstance(getResources().getString(R.string.goong_api_url)).create(IApiService.class);
        Call<PlaceDetailResponse> call = service.getPlaceDetail(placeId, getResources().getString(R.string.goong_api_key));
        call.enqueue(new Callback<PlaceDetailResponse>() {
            @Override
            public void onResponse(Call<PlaceDetailResponse> call, Response<PlaceDetailResponse> response) {
                //TODO: Hiển thị đánh marker cho điểm theo tọa độ
                if (response != null && response.isSuccessful()) {
                    PlaceDetailResponse data = response.body();
                    if (data != null && data.getResult() != null && data.getResult().getGeometry() != null) {
                        Location location = data.getResult().getGeometry().getLocation();
                        try {
                            LatLng center = new LatLng(Double.parseDouble(location.getLat()), Double.parseDouble(location.getLng()));
                            drawCircleLineAndMarker(center, style);
                            selectedLocation = center;
                            mapboxMap.clear();
                            mapboxMap.addMarker(new MarkerOptions()
                                    .position(center));
                        } catch (Exception e) {
                            Toast.makeText(MainActivity.this, "Parsing is error!!!", Toast.LENGTH_SHORT).show();
                        }
                    }
                }
            }

            @Override
            public void onFailure(Call<PlaceDetailResponse> call, Throwable t) {
                Toast.makeText(MainActivity.this, "FetchDetailLocation is error!!!", Toast.LENGTH_SHORT).show();
            }
        });
    }

    private void fetchDirections(LatLng start, LatLng end, Style style) {
        String origin = start.getLatitude() + "," + start.getLongitude(),
                destination = end.getLatitude() + "," + end.getLongitude();
        IApiService service = RetrofitInstance.getRetrofitInstance(getResources().getString(R.string.goong_api_url)).create(IApiService.class);
        Call<DirectionResponse> call = service.getDirection(origin, destination, "car", getResources().getString(R.string.goong_api_key));
        call.enqueue(new Callback<DirectionResponse>() {
            @Override
            public void onResponse(Call<DirectionResponse> call, Response<DirectionResponse> response) {
                //Gọi API lấy danh sách các điểm
                if (response != null && response.isSuccessful()) {
                    DirectionResponse data = response.body();
                    if (data != null && data.getRoutes() != null) {
                        Route route = data.getRoutes().get(0);
                        if (route.getOverviewPolyline() != null && route.getOverviewPolyline() != null &&
                                route.getOverviewPolyline().getPoints() != null) {
                            String geometry = route.getOverviewPolyline().getPoints();
                            List<Point> points = LineString.fromPolyline(geometry, 5).coordinates();
                            drawLineBetweenTwoPoint(points, style);
                            mapboxMap.clear();
                            mapboxMap.addMarker(new MarkerOptions()
                                    .position(start));
                            mapboxMap.addMarker(new MarkerOptions()
                                    .position(end));
                            bound(start, end);
                        }
                    }
                }
            }

            @Override
            public void onFailure(Call<DirectionResponse> call, Throwable t) {
                Log.d("Call error", "Call error");
            }
        });
    }

    public void bound(LatLng start, LatLng end) {
        LatLngBounds latLngBounds = new LatLngBounds.Builder()
                .include(start) // Northeast
                .include(end) // Southwest
                .build();

        mapboxMap.easeCamera(CameraUpdateFactory.newLatLngBounds(latLngBounds, 20), 5000);
    }

    public void drawLineBetweenTwoPoint(List<Point> points, Style style) {
        LineString lineString = LineString.fromLngLats(points);
        Feature feature = Feature.fromGeometry(lineString);
        GeoJsonSource geoJsonSource = new GeoJsonSource("line-source", feature);
        style.addSource(geoJsonSource);
        style.addLayer(new LineLayer("linelayer", "line-source")
                .withProperties(PropertyFactory.lineCap(Property.LINE_CAP_SQUARE),
                        PropertyFactory.lineJoin(Property.LINE_JOIN_MITER),
                        PropertyFactory.lineOpacity(.7f),
                        PropertyFactory.lineWidth(7f),
                        PropertyFactory.lineColor(Color.parseColor("#3887be"))));
    }

    public void drawCircleLineAndMarker(LatLng center, Style style) {
        List<Point> points = this.getCircleLatLng(center, 300);
        //Vẽ fill color hình tròn
        Feature circlePolygon = Feature.fromGeometry(createPolygonFromPoints(points));
        style.addSource(new GeoJsonSource("circle_polygon", circlePolygon));
        style.addLayer(new FillLayer("polygon-layer", "circle_polygon")
                .withProperties(
                        PropertyFactory.fillOpacity(0.2f),
                        PropertyFactory.fillColor(Color.parseColor("#588888"))
                ));
        //Vẽ line xung quanh
        style.addLayer(new LineLayer("circle_layer", "circle_source")
                .withProperties(PropertyFactory.lineCap(Property.LINE_CAP_SQUARE),
                        PropertyFactory.lineJoin(Property.LINE_JOIN_MITER),
                        PropertyFactory.lineOpacity(.7f),
                        PropertyFactory.lineWidth(0.3f),
                        PropertyFactory.lineColor(Color.parseColor("#588888"))));

        CameraPosition position = new CameraPosition.Builder()
                .target(center)
                .zoom(15)
                .tilt(20)
                .build();
        mapboxMap.animateCamera(CameraUpdateFactory.newCameraPosition(position), 1200);

    }

    public static Polygon createPolygonFromPoints(List<Point> points) {
        List<List<Point>> polygonCoordinates = new ArrayList<>();
        polygonCoordinates.add(points);
        return Polygon.fromLngLats(polygonCoordinates);
    }

    public List<Point> getCircleLatLng(LatLng center, double radius) {
        List<Point> result = new ArrayList<>();
        int points = 64;
        double[][] coordinates = new double[points + 1][2];
        for (int i = 0; i < points; i++) {
            double angle = i * (2 * Math.PI / points);
            double dx = radius * Math.cos(angle);
            double dy = radius * Math.sin(angle);
            coordinates[i] = new double[]{
                    center.getLongitude() + (dx / 110540f),
                    center.getLatitude() + (dy / 110540f)
            };
        }
        coordinates[points] = coordinates[0]; // Close the circle
        for (double[] coordinate : coordinates) {
            result.add(Point.fromLngLat(coordinate[0], coordinate[1]));
        }
        return result;
    }

    @Override
    protected void onStart() {
        super.onStart();
        mapView.onStart();
    }

    @Override
    protected void onResume() {
        super.onResume();
        mapView.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
        mapView.onPause();
    }

    @Override
    protected void onStop() {
        super.onStop();
        mapView.onStop();
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        mapView.onSaveInstanceState(outState);
    }

    @Override
    public void onLowMemory() {
        super.onLowMemory();
        mapView.onLowMemory();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mapView.onDestroy();
    }
}
